#The path for files.
path = "c:\\hry\\BotArena\\Decks\\experiment\\commander2\\"

#The list of all sets, in chronological order.
sets = ["UTD", "ARN", "ATQ", "REV", "LEG", "DRK", "FEM", "4TH", "PRO", "ICE", "HML", "ALL", "MRG", "VIS", "5TH", "POR", "WTH", "TMP", "STH", "P02", "EXO", "USG", "ULG", "6TH", "PTK", "UDS", "S99", "MMQ", "NMS", "PCY", "S00", "INV", "PLS", "7TH", "APC", "ODY", "TOR", "JUD", "ONS", "LGN", "SCG", "8TH", "MRD", "DST", "5DN", "CHK", "BOK", "SOK", "9TH", "RAV", "GPT", "DIS", "CSP", "TSP", "PLC", "FUT", "10TH", "LRW", "MOR", "SHM", "EVE", "ALA", "CON", "ARB", "M10", "ZEN", "WWK", "ROE", "ARC", "M11", "SOM", "MBS", "NPH", "CMD", "M12", "ISD"]

#The list of sets that don't contain any nonbasic lands (still work in progress). It's used to ensure that the program won't try to open nonexistent land files.
no_land_sets = ["5DN", "ARB", "LGN", "P02", "POR", "PRO", "PTK", "S00", "S99"]

#The list of sets with snow. Any deck containing one of these sets will split the basics 50:50 between normal and snow.
snow_sets = ["ICE", "ALL", "CSP"]

#List of cards whose existence will cause a sideboard to be added:
sideboard_cards = ["Death Wish", "Burning Wish", "Cunning Wish", "Golden Wish", "Living Wish", "Glittering Wish"]

#This takes a line like "BRG: Armorer Guildmage (9312)" and outputs a list of color data ("BRG"), card name ("Armorer Guildmage") and the complete card entry ("Armorer Guildmage (9312)").
def separate_1(l):
    a, b = l.index(":"), l.index("(")
    return [l[:a], l[(a + 2):(b - 1)], l[(a + 2):]]

#This takes a line like "Akroma, Angel of Wrath (14400)" and outputs the card name ("Akroma, Angel of Wrath"). Used for commanders and the contents of colorless land file.
def separate_2(l):
    a = l.index("(")
    return l[:(a - 1)]

#This takes the color data and outputs the list of "forbidden symbols". Any of these forbidden symbols in the color data of a card will stop it from being added to deck.
#The symbols are colors, plus numbers 1 to 5. A number in color data means that only a deck with this many or more colors can use the card. For example, Armillary Sphere gets "1", since colorless decks have no basic lands and so can't make use of Armillary Sphere.
#Quirion Dryad gets "G2", as it needs you to play a second color to be fully utilized, but it doesn't care which color it is. A monocolored deck would have "2" in the list of forbidden symbols, so it would throw Quirion Dryad out.
def forbidden_symbols(color_string):
    if color_string != "C":
        color_length = len(color_string)-1
        symbols = "WUBRG"
        output = ""
        for i in symbols:
            if color_string.count(i) == 0:
                output += i
        for j in range(color_length, 5):
            output += str(j)
        return output
    else:
        return "WUBRG12345"

#This transforms set code into a code for card file.
def set_file(x):
    return (path + "data\\EXP - " + x + ".txt")

#This transforms set code into a code for land file.
def set_land_file(x):
    return (path + "data\\LND - " + x + ".txt")

#This combines the name of a commander and codes of the two sets into a filename.
def commander_deck_name(commander, set1, set2):
    return (path + "decks\\" + commander + " - " + set1 + " + " + set2 + ".txt")

#This checks color data extracted from a line to see if they contain any forbidden symbol. It returns true if no forbidden symbol is found.
def card_allowed(forbidden_string, color_data):
    allowed = True
    for i in color_data:
        if forbidden_string.count(i) > 0:
            allowed = False
            break
    return allowed

#This reads a line from infile (skipping empty lines). It writes the card mentioned there to the output file if it's not in card_list (which would mean that another version was already written, or that the card is a commander) and if the card_allowed function verifies it's compatible.
#Returns false when it hits the end of file.
def card_test(infile, forbidden_string, card_list, outfile):
    line = infile.readline()
    while line == "\n":
        line = infile.readline()
    if len(line) > 0:
        line_sep = separate_1(line)
        if card_list.count(line_sep[1]) == 0 and card_allowed(forbidden_string, line_sep[0]):
            card_list.append(line_sep[1])
            outfile.write(line_sep[2])
        return True
    else:
        return False

#This is an alternative test that uses colorless lands file. Since cards in this file don't use color data, it leaves the compatibility check out.
def card_test_colorless_lands(infile, card_list, outfile):
    line = infile.readline()
    while len(line) > 0 and (line == "\n" or line[0] == "-"):
        line = infile.readline()
    if len(line) > 0:
        line_sep = separate_2(line)
        if card_list.count(line_sep) == 0:
            card_list.append(line_sep)
            outfile.write(line)
        return True
    else:
        return False

#This reads commander data from master file. Each commander is on three lines, first is the card code, second is the set code, third is the color data.
#The data it reads are stored in a variable; the output is, once again, true or false based on whether it can continue reading or whether it hit the end of file.
def read_commander_data(infile):
    line1 = infile.readline()
    while line1 == "\n":
        line1 = infile.readline()
    if len(line1) > 0:
        line2 = infile.readline()
        line3 = infile.readline()
        return [line1, line2, line3]
    else:
        return []

#The main procedure which creates a commander deck from the commander card data, two set codes, and color data. I will comment throughout, as it's complex.
def create_commander_deck(commander, set1, set2, color_data, output):
    forbidden_string = forbidden_symbols(color_data)
    #the "snowy" variable keeps track of whether the deck uses a snow set (Ice Age, Alliances or Coldsnap).
    snowy = False
    file1 = open(set_file(set1),"r")
    comname = commander_deck_name(separate_2(commander), set1, set2)
    file_out = open(comname,"w")
    if snow_sets.count(set1) > 0:
        snowy = True
    k = True
    #The card_list variable will store the list of all cards to screen out duplicates. It starts with the commander, so it can't be added to the deck.
    card_list = [separate_2(commander)]
    file_out.write("#LIFE 40\n\nCommander\n\n")
    file_out.write("// Set 1: " + set1 + "\n\n")
    #This is the main loop which is used several times with some variations. It should go through the whole input file, write all eligible cards to the output file, and stop when it gets to the end of file.
    while k:
        k = card_test(file1, forbidden_string, card_list, file_out)
    file1.close()
    file_out.write("\n// Set 2: " + set2 + "\n\n")
    file2 = open(set_file(set2),"r")
    if snow_sets.count(set2) > 0:
        snowy = True
    k = True
    while k:
        k = card_test(file2, forbidden_string, card_list, file_out)
    file2.close()
    #This part comes into play if the trawl through both input files fails to find 60 cards (counting the commander). This would put the size of resulting deck to less than 100 cards. In this case, sets chronologically after set2 start getting added, one by one, until the threshold is passed. This is why the sets list is in chronological order.
    setindex = sets.index(set2)
    extra = 0
    while len(card_list) < 60:
        setindex += 1
        extra += 1
        if setindex >= len(sets):
            setindex = 0
        file_out.write("\n// Additional set " + str(extra) + ": " + sets[setindex] + " (card count: " + str(len(card_list)) + ")\n\n")
        file2 = open(set_file(sets[setindex]))
        if snow_sets.count(sets[setindex]) > 0:
            snowy = True
        k = True
        while k:
            k = card_test(file2, forbidden_string, card_list, file_out)
        file2.close()
    #I presume that the standard commander deck should be 60 nonland cards/40 lands. Since the number of nonland cards can be higher, I opt to preserve the ratio. This operation will set land_goal to 2/3 the number of nonland cards, rounded up.
    land_goal = (len(card_list) * 2 + 2) // 3
    land_list = []
    #Now we repeat the file trawls. The difference is that the sets must be checked against the no_land_sets list to ensure that a file with the given name exists, and only the two input sets are checked.
    file_out.write("\n// Lands:\n\n")
    if no_land_sets.count(set1) == 0:
        file1 = open(set_land_file(set1),"r")
        k = True
        while k:
            k = card_test(file1, forbidden_string, land_list, file_out)
        file1.close()
    #Once the first set is checked, it's time to check the special case of colorless commanders. Since colorless commanders can't play basic lands, they draw lands from the pool of "Colorless Lands.txt". They can potentially get lands from any set.
    if color_data != "C":
        if no_land_sets.count(set2) == 0:
            file2 = open(set_land_file(set2),"r")
            k = True
            while k:
                k = card_test(file2, forbidden_string, land_list, file_out)
            file2.close()
        #For normal (colored) commanders, the basics are added. First the total number of lands is reduced by the number of nonbasics added to the deck, then it's evenly divided (rounded up) among all colors the commander has, which is the length of his/her/its color data.
        color_number = len(color_data)
        basic_number = (land_goal - len(land_list) + color_number - 1) // color_number
        dic = {"W": "Plains", "U": "Island", "B": "Swamp", "R": "Mountain", "G": "Forest"}
        #Here's where the "snowy" variables pays off. If the deck was determined to contain snow sets, only half of the basics added (rounded up) will be normal; half will be snow.
        if snowy:
            for i in color_data:
                file_out.write(str((basic_number + 1) // 2) + " " + dic[i] + "\n")
                file_out.write(str(basic_number // 2) + " Snow-Covered " + dic[i] + "\n")
        else:
            for i in color_data:
                file_out.write(str(basic_number) + " " + dic[i] + "\n")
    #On the other hand, colorless commanders use this part. First it finds the appropriate point in the "Colorless Lands.txt" to start (at the second set).
    #Then it goes to the end of the file, adding all lands that weren't added before. If the quota is still not met, it goes through the list again, from beginning. Then it's done, even if the number of lands is still too low.
    else:
        file2 = open(path + "data\\Colorless Lands.txt")
        l = ""
        while l != ("-" + set2 + "\n"):
            l = file2.readline()
        k = True
        while k and len(land_list) < land_goal:
            k = card_test_colorless_lands(file2, land_list, file_out)
        file2.close()
        if len(land_list) < land_goal:
            file2 = open(path + "data\\Colorless Lands.txt")
            k = True
            while k and len(land_list) < land_goal:
                k = card_test_colorless_lands(file2, land_list, file_out)
            file2.close()
    file_out.write("\n#STARTEMBLEMS\n\n" + commander + "\n")
    #Sideboard checks start:
    sideboard_list = []
    past_count = 0
    for si in sideboard_cards:
        if card_list.count(si) > 0:
            sidefile = open(path + "data\\SID - " + si + ".txt", "r")
            if len(sideboard_list) == 0:
                file_out.write("#SIDEBOARD\n\n")
            elif past_count != len(sideboard_list):
                file_out.write("\n")
            past_count = len(sideboard_list)
            k = True
            while k:
                k = card_test(sidefile, forbidden_string, sideboard_list, file_out)
            sidefile.close()
    file_out.close()
    if len(sideboard_list) == 0:
        out_string = comname[len(path + "decks\\"):] + " closed. " + str(extra) + " extra sets, " + str(len(card_list)) + " nonland cards. Land goal: " + str(land_goal)
    else:
        out_string = comname[len(path + "decks\\"):] + " closed. " + str(extra) + " extra sets, " + str(len(card_list)) + " nonland cards. Land goal: " + str(land_goal) + ". Sideboard size: " + str(len(sideboard_list))
    print(out_string)
    output.write(out_string + "\n")

#This goes through all sets and creates commander decks with given commander. It skips the commander's set.
def model_commander(commander, set, color_data, output):
    for i in sets:
        if i != set:
            create_commander_deck(commander, set, i, color_data, output)

#This reads through the master file and creates decks for every commander mentioned there.
def create_all_commanders(masterfile, output):
    k = True
    while k:
        q = read_commander_data(masterfile)
        if q == []:
            k = False
        else:
            k = True
        if k:
            print("Commander data read: " + q[0] +  q[1] + q[2])
            model_commander(q[0], q[1][:-1], q[2][:-1], output)

masterfile = open(path + "data\\Master.txt", "r")
output = open (path + "data\\output.txt", "w")
create_all_commanders(masterfile, output)
masterfile.close()
output.close()
